"""Model Registry REST API.

REST API for Model Registry to create and manage ML model metadata

The version of the OpenAPI document: v1alpha3
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

import unittest

from mr_openapi.models.model_version_create import ModelVersionCreate


class TestModelVersionCreate(unittest.TestCase):
    """ModelVersionCreate unit test stubs."""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> ModelVersionCreate:
        """Test ModelVersionCreate
        include_option is a boolean, when False only required
        params are included, when True both required and
        optional params are included.
        """
        # uncomment below to create an instance of `ModelVersionCreate`
        """
        model = ModelVersionCreate()
        if include_optional:
            return ModelVersionCreate(
                custom_properties = {
                    'key' : null
                    },
                description = '',
                external_id = '',
                name = '',
                state = 'LIVE',
                author = '',
                registered_model_id = ''
            )
        else:
            return ModelVersionCreate(
                registered_model_id = '',
        )
        """

    def testModelVersionCreate(self):
        """Test ModelVersionCreate."""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)


if __name__ == "__main__":
    unittest.main()
