"""Model Registry REST API.

REST API for Model Registry to create and manage ML model metadata

The version of the OpenAPI document: v1alpha3
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

import unittest


class TestRegisteredModelState(unittest.TestCase):
    """RegisteredModelState unit test stubs."""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testRegisteredModelState(self):
        """Test RegisteredModelState."""
        # inst = RegisteredModelState()


if __name__ == "__main__":
    unittest.main()
