"""Model Registry REST API.

REST API for Model Registry to create and manage ML model metadata

The version of the OpenAPI document: v1alpha3
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
from enum import Enum

from typing_extensions import Self


class InferenceServiceState(str, Enum):
    """- DEPLOYED: A state indicating that the `InferenceService` should be deployed. - UNDEPLOYED: A state indicating that the `InferenceService` should be un-deployed. The state indicates the desired state of inference service. See the associated `ServeModel` for the actual status of service deployment action."""

    """
    allowed enum values
    """
    DEPLOYED = "DEPLOYED"
    UNDEPLOYED = "UNDEPLOYED"

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of InferenceServiceState from a JSON string."""
        return cls(json.loads(json_str))
